DEFAULT_MODULES = [
    'calibration',
    'cluster',
    'compose',
    'covariance',
    'cross_decomposition',
    'decomposition',
    'discriminant_analysis',
    'ensemble',
    'feature_extraction',
    'feature_selection',
    'gaussian_process',
    'impute',
    'isotonic',
    'kernel_approximation',
    'kernel_ridge',
    'linear_model',
    'manifold',
    'mixture',
    'naive_bayes',
    'neighbors',
    'neural_network',
    'preprocessing',
    'random_projection',
    'semi_supervised',
    'svm',
    'tree'
]